from rest_framework import serializers
from netbox.api.serializers import NetBoxModelSerializer, WritableNestedSerializer
from ..models import VLANGroupSet


class VLANGroupSetSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name=f'plugins-api:netbox_vlan_manager-api:vlangroupset-detail'
    )
    vlan_group_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = VLANGroupSet
        fields = (
            'id',
            'url',
            'display',
            'name',
            'description',
            'comments',
            'tags',
            'custom_fields',
            'created',
            'last_updated',
            'vlan_group_count',
        )


class NestedVLANGroupSetSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name=f'plugins-api:netbox_vlan_manager-api:vlangroupset-detail'
    )
    vlan_group_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = VLANGroupSet
        fields = (
            'id',
            'url',
            'display',
            'name',
            'description',
            'vlan_group_count',
        )


class AvailableVLANSerializer(serializers.Serializer):
    vid = serializers.IntegerField(read_only=True)
    group_set = NestedVLANGroupSetSerializer(read_only=True)

    def to_representation(self, instance):
        return {
            'vid': instance,
            'group_set': NestedVLANGroupSetSerializer(
                self.context['group_set'],
                context={'request': self.context['request']}
            ).data,
        }



from extras.plugins import PluginMenuItem, PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

vlan_group_manager_buttons = [
    PluginMenuButton(
        link=f'plugins:netbox_vlan_manager:vlangroupset_add',
        title='Add',
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN
    )
]

menu_items = (
    PluginMenuItem(
        link=f'plugins:netbox_vlan_manager:vlangroupset_list',
        link_text='VLAN Group Set',
        buttons=vlan_group_manager_buttons
    ),
)



from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField, DynamicModelMultipleChoiceField
from ipam.models import VLANGroup
from .models import VLANGroupSet


class VLANGroupSetForm(NetBoxModelForm):
    vlan_groups = DynamicModelMultipleChoiceField(
        queryset=VLANGroup.objects.all(),
        required=False
    )
    comments = CommentField()

    class Meta:
        model = VLANGroupSet
        fields = (
            'name',
            'vlan_groups',
            'description',
            'comments',
            'tags'
        )



from django.db.models import Count
from django.shortcuts import get_object_or_404
from rest_framework.response import Response
from rest_framework.views import APIView
from netbox.config import get_config
from netbox.api.viewsets import NetBoxModelViewSet
from .serializers import VLANGroupSetSerializer, AvailableVLANSerializer
from ..models import VLANGroupSet


def get_results_limit(request):
    config = get_config()
    try:
        limit = int(request.query_params.get(
            'limit', config.PAGINATE_COUNT)) or config.MAX_PAGE_SIZE
    except ValueError:
        limit = config.PAGINATE_COUNT
    if config.MAX_PAGE_SIZE:
        limit = min(limit, config.MAX_PAGE_SIZE)

    return limit


class VLANGroupSetListViewSet(NetBoxModelViewSet):
    queryset = VLANGroupSet.objects.prefetch_related('tags').annotate(
        vlan_group_count=Count('vlan_groups')
    )
    serializer_class = VLANGroupSetSerializer


class AvailableVLANsView(APIView):
    queryset = VLANGroupSet.objects

    def get(self, request, pk):
        vlan_group_set = get_object_or_404(VLANGroupSet, pk=pk)
        limit = get_results_limit(request)

        available_vlans = vlan_group_set.get_available_vids()[:limit]
        serializer = AvailableVLANSerializer(available_vlans, many=True, context={
            'request': request,
            'group_set': vlan_group_set,
        })

        return Response(serializer.data)



from django import template
register = template.Library()


@register.filter
def get_vlan_by_group(record, vlan_group):
    vlan = ([x for x in record['vlans'] if x.group == vlan_group] or [None])[0]
    return vlan



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel
from ipam.models import VLAN


class VLANGroupSet(NetBoxModel):
    name = models.CharField(
        max_length=100,
        unique=True
    )
    vlan_groups = models.ManyToManyField(
        to='ipam.VLANGroup',
        related_name='vlan_group_sets',
        blank=True
    )
    description = models.CharField(
        max_length=200,
        blank=True
    )
    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse(f'plugins:netbox_vlan_manager:vlangroupset', kwargs={'pk': self.pk})

    @property
    def min_vid(self):
        return min(self.vlan_groups.all(), key=(lambda x: x.min_vid)).min_vid

    @property
    def max_vid(self):
        return max(self.vlan_groups.all(), key=(lambda x: x.max_vid)).max_vid

    @property
    def vlans(self):
        vlan_groups = self.vlan_groups.all()
        group_vlans = VLAN.objects.filter(
            group__in=vlan_groups).select_related('group')

        vlan_group_vlans = []
        for vid in range(self.min_vid, self.max_vid + 1):
            item = {}
            item['vid'] = vid
            vlans = [x for x in group_vlans if x.vid == vid]
            item['vlans'] = vlans
            item['status'] = 'Available' if not vlans else 'In Use'
            vlan_group_vlans.append(item)
        return vlan_group_vlans

    def get_available_vids(self):
        available_vlans = set([x['vid']
                               for x in self.vlans if x['status'] == 'Available'])

        return sorted(available_vlans)

    def get_next_available_vid(self):
        available_vids = self.get_available_vids()
        if available_vids:
            return available_vids[0]
        return None



from django.urls import path
from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'netbox_vlan_manager'

router = NetBoxRouter()
router.register('vlan-group-sets', views.VLANGroupSetListViewSet)

urlpatterns = [
    path(
        'vlan-group-sets/<int:pk>/available-vlans/',
        views.AvailableVLANsView.as_view(),
        name='vlangroupset-available-vlans'
    ),
]
urlpatterns += router.urls
